/* 
 * $Id: MapBasedSftpProducer.java 3373 2011-01-05 01:34:44Z summer_h $
 * $URL: https://open.jira.com/svn/IKASAN/tags/ikasan-0.8.0/connector-sftp/client/src/main/java/org/ikasan/endpoint/sftp/producer/type/MapBasedSftpProducer.java $
 *
 * ====================================================================
 * Ikasan Enterprise Integration Platform
 * 
 * Distributed under the Modified BSD License.
 * Copyright notice: The copyright for this software and a full listing 
 * of individual contributors are as shown in the packaged copyright.txt 
 * file. 
 * 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *
 *  - Neither the name of the ORGANIZATION nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
package org.ikasan.endpoint.sftp.producer.type;

import java.io.InputStream;
import java.util.Map;

import javax.resource.ResourceException;

import org.ikasan.client.FileTransferConnectionTemplate;
import org.ikasan.endpoint.sftp.producer.SftpProducerConfiguration;
import org.ikasan.spec.endpoint.Producer;


/**
 * Sftp producer based on a Map<String,InputStream>.
 * This implementation is provided as a stepping stone to moving away from
 * the FileTransfer client approach. Ideally, this class should be interacting
 * with the SFTP API and not another File Transfer client layer. TODO!
 * @author Ikasan Development Team
 */
public class MapBasedSftpProducer implements Producer<Map<String,InputStream>>
{
    /** existing template */
    protected FileTransferConnectionTemplate fileTransferConnectionTemplate;
    
    /** configuration */
    protected SftpProducerConfiguration sftpProducerConfiguration;

    /**
     * Constructor
     * @param fileTransferConnectionTemplate
     * @param sftpProducerConfiguration
     */
    public MapBasedSftpProducer(FileTransferConnectionTemplate fileTransferConnectionTemplate, SftpProducerConfiguration sftpProducerConfiguration)
    {
        this.fileTransferConnectionTemplate = fileTransferConnectionTemplate;
        if(fileTransferConnectionTemplate == null)
        {
            throw new IllegalArgumentException("fileTransferConnectionTemplate cannot be 'null'");
        }

        this.sftpProducerConfiguration = sftpProducerConfiguration;
        if(sftpProducerConfiguration == null)
        {
            throw new IllegalArgumentException("sftpProducerConfiguration cannot be 'null'");
        }
    }
    
    /**
     * Invoke delivery based on passing a map of String (filename) and InputStream (content) pairs
     * to allow compatibility with the existing FileTransferConnectionTemplate.
     * @param Map<String,InputStream>
     */
    public void invoke(Map<String,InputStream> filenameContentPairs) throws ResourceException
    {
        for(Map.Entry<String,InputStream> filenameContent : filenameContentPairs.entrySet())
        {
            this.fileTransferConnectionTemplate.deliverInputStream(filenameContent.getValue(), filenameContent.getKey(), 
                    sftpProducerConfiguration.getOutputDirectory(), sftpProducerConfiguration.getOverwrite(), 
                    sftpProducerConfiguration.getRenameExtension(), sftpProducerConfiguration.getChecksumDelivered(), 
                    sftpProducerConfiguration.getUnzip(), sftpProducerConfiguration.getCreateParentDirectory(), sftpProducerConfiguration.getTempFileName());
        }        
    }
}
